/*    form.h
 *
 *    Copyright (C) 1991, 1992, 1993, 2000, 2004 by Larry Wall and others
 *
 *    You may distribute under the terms of either the GNU General Public
 *    License or the Artistic License, as specified in the README file.
 *
 */

#define FF_END          0
#define FF_LINEMARK     1
#define FF_LITERAL      2
#define FF_SKIP         3
#define FF_FETCH        4
#define FF_CHECKNL      5
#define FF_CHECKCHOP    6
#define FF_SPACE        7
#define FF_HALFSPACE    8
#define FF_ITEM         9
#define FF_CHOP         10
#define FF_LINEGLOB     11
#define FF_DECIMAL      12
#define FF_NEWLINE      13
#define FF_BLANK        14
#define FF_MORE         15
#define FF_0DECIMAL     16
#define FF_LINESNGL     17
