/* -*- buffer-read-only: t -*-
   !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
   This file is built by opcode.pl from its data.  Any changes made here
   will be lost!
*/

PERL_CKDEF(Perl_ck_anoncode)
PERL_CKDEF(Perl_ck_bitop)
PERL_CKDEF(Perl_ck_chdir)
PERL_CKDEF(Perl_ck_concat)
PERL_CKDEF(Perl_ck_defined)
PERL_CKDEF(Perl_ck_delete)
PERL_CKDEF(Perl_ck_die)
PERL_CKDEF(Perl_ck_eof)
PERL_CKDEF(Perl_ck_eval)
PERL_CKDEF(Perl_ck_exec)
PERL_CKDEF(Perl_ck_exists)
PERL_CKDEF(Perl_ck_exit)
PERL_CKDEF(Perl_ck_ftst)
PERL_CKDEF(Perl_ck_fun)
PERL_CKDEF(Perl_ck_glob)
PERL_CKDEF(Perl_ck_grep)
PERL_CKDEF(Perl_ck_index)
PERL_CKDEF(Perl_ck_join)
PERL_CKDEF(Perl_ck_lengthconst)
PERL_CKDEF(Perl_ck_lfun)
PERL_CKDEF(Perl_ck_listiob)
PERL_CKDEF(Perl_ck_match)
PERL_CKDEF(Perl_ck_method)
PERL_CKDEF(Perl_ck_null)
PERL_CKDEF(Perl_ck_open)
PERL_CKDEF(Perl_ck_repeat)
PERL_CKDEF(Perl_ck_require)
PERL_CKDEF(Perl_ck_return)
PERL_CKDEF(Perl_ck_rfun)
PERL_CKDEF(Perl_ck_rvconst)
PERL_CKDEF(Perl_ck_sassign)
PERL_CKDEF(Perl_ck_say)
PERL_CKDEF(Perl_ck_select)
PERL_CKDEF(Perl_ck_shift)
PERL_CKDEF(Perl_ck_smartmatch)
PERL_CKDEF(Perl_ck_sort)
PERL_CKDEF(Perl_ck_spair)
PERL_CKDEF(Perl_ck_split)
PERL_CKDEF(Perl_ck_subr)
PERL_CKDEF(Perl_ck_substr)
PERL_CKDEF(Perl_ck_svconst)
PERL_CKDEF(Perl_ck_trunc)
PERL_CKDEF(Perl_ck_unpack)


PERL_PPDEF(Perl_pp_null)
PERL_PPDEF(Perl_pp_stub)
PERL_PPDEF(Perl_pp_scalar)
PERL_PPDEF(Perl_pp_pushmark)
PERL_PPDEF(Perl_pp_wantarray)
PERL_PPDEF(Perl_pp_const)
PERL_PPDEF(Perl_pp_gvsv)
PERL_PPDEF(Perl_pp_gv)
PERL_PPDEF(Perl_pp_gelem)
PERL_PPDEF(Perl_pp_padsv)
PERL_PPDEF(Perl_pp_padav)
PERL_PPDEF(Perl_pp_padhv)
PERL_PPDEF(Perl_pp_padany)
PERL_PPDEF(Perl_pp_pushre)
PERL_PPDEF(Perl_pp_rv2gv)
PERL_PPDEF(Perl_pp_rv2sv)
PERL_PPDEF(Perl_pp_av2arylen)
PERL_PPDEF(Perl_pp_rv2cv)
PERL_PPDEF(Perl_pp_anoncode)
PERL_PPDEF(Perl_pp_prototype)
PERL_PPDEF(Perl_pp_refgen)
PERL_PPDEF(Perl_pp_srefgen)
PERL_PPDEF(Perl_pp_ref)
PERL_PPDEF(Perl_pp_bless)
PERL_PPDEF(Perl_pp_backtick)
PERL_PPDEF(Perl_pp_glob)
PERL_PPDEF(Perl_pp_readline)
PERL_PPDEF(Perl_pp_rcatline)
PERL_PPDEF(Perl_pp_regcmaybe)
PERL_PPDEF(Perl_pp_regcreset)
PERL_PPDEF(Perl_pp_regcomp)
PERL_PPDEF(Perl_pp_match)
PERL_PPDEF(Perl_pp_qr)
PERL_PPDEF(Perl_pp_subst)
PERL_PPDEF(Perl_pp_substcont)
PERL_PPDEF(Perl_pp_trans)
PERL_PPDEF(Perl_pp_sassign)
PERL_PPDEF(Perl_pp_aassign)
PERL_PPDEF(Perl_pp_chop)
PERL_PPDEF(Perl_pp_schop)
PERL_PPDEF(Perl_pp_chomp)
PERL_PPDEF(Perl_pp_schomp)
PERL_PPDEF(Perl_pp_defined)
PERL_PPDEF(Perl_pp_undef)
PERL_PPDEF(Perl_pp_study)
PERL_PPDEF(Perl_pp_pos)
PERL_PPDEF(Perl_pp_preinc)
PERL_PPDEF(Perl_pp_predec)
PERL_PPDEF(Perl_pp_postinc)
PERL_PPDEF(Perl_pp_postdec)
PERL_PPDEF(Perl_pp_pow)
PERL_PPDEF(Perl_pp_multiply)
PERL_PPDEF(Perl_pp_i_multiply)
PERL_PPDEF(Perl_pp_divide)
PERL_PPDEF(Perl_pp_i_divide)
PERL_PPDEF(Perl_pp_modulo)
PERL_PPDEF(Perl_pp_i_modulo)
PERL_PPDEF(Perl_pp_repeat)
PERL_PPDEF(Perl_pp_add)
PERL_PPDEF(Perl_pp_i_add)
PERL_PPDEF(Perl_pp_subtract)
PERL_PPDEF(Perl_pp_i_subtract)
PERL_PPDEF(Perl_pp_concat)
PERL_PPDEF(Perl_pp_stringify)
PERL_PPDEF(Perl_pp_left_shift)
PERL_PPDEF(Perl_pp_right_shift)
PERL_PPDEF(Perl_pp_lt)
PERL_PPDEF(Perl_pp_i_lt)
PERL_PPDEF(Perl_pp_gt)
PERL_PPDEF(Perl_pp_i_gt)
PERL_PPDEF(Perl_pp_le)
PERL_PPDEF(Perl_pp_i_le)
PERL_PPDEF(Perl_pp_ge)
PERL_PPDEF(Perl_pp_i_ge)
PERL_PPDEF(Perl_pp_eq)
PERL_PPDEF(Perl_pp_i_eq)
PERL_PPDEF(Perl_pp_ne)
PERL_PPDEF(Perl_pp_i_ne)
PERL_PPDEF(Perl_pp_ncmp)
PERL_PPDEF(Perl_pp_i_ncmp)
PERL_PPDEF(Perl_pp_slt)
PERL_PPDEF(Perl_pp_sgt)
PERL_PPDEF(Perl_pp_sle)
PERL_PPDEF(Perl_pp_sge)
PERL_PPDEF(Perl_pp_seq)
PERL_PPDEF(Perl_pp_sne)
PERL_PPDEF(Perl_pp_scmp)
PERL_PPDEF(Perl_pp_bit_and)
PERL_PPDEF(Perl_pp_bit_xor)
PERL_PPDEF(Perl_pp_bit_or)
PERL_PPDEF(Perl_pp_negate)
PERL_PPDEF(Perl_pp_i_negate)
PERL_PPDEF(Perl_pp_not)
PERL_PPDEF(Perl_pp_complement)
PERL_PPDEF(Perl_pp_atan2)
PERL_PPDEF(Perl_pp_sin)
PERL_PPDEF(Perl_pp_cos)
PERL_PPDEF(Perl_pp_rand)
PERL_PPDEF(Perl_pp_srand)
PERL_PPDEF(Perl_pp_exp)
PERL_PPDEF(Perl_pp_log)
PERL_PPDEF(Perl_pp_sqrt)
PERL_PPDEF(Perl_pp_int)
PERL_PPDEF(Perl_pp_hex)
PERL_PPDEF(Perl_pp_oct)
PERL_PPDEF(Perl_pp_abs)
PERL_PPDEF(Perl_pp_length)
PERL_PPDEF(Perl_pp_substr)
PERL_PPDEF(Perl_pp_vec)
PERL_PPDEF(Perl_pp_index)
PERL_PPDEF(Perl_pp_rindex)
PERL_PPDEF(Perl_pp_sprintf)
PERL_PPDEF(Perl_pp_formline)
PERL_PPDEF(Perl_pp_ord)
PERL_PPDEF(Perl_pp_chr)
PERL_PPDEF(Perl_pp_crypt)
PERL_PPDEF(Perl_pp_ucfirst)
PERL_PPDEF(Perl_pp_lcfirst)
PERL_PPDEF(Perl_pp_uc)
PERL_PPDEF(Perl_pp_lc)
PERL_PPDEF(Perl_pp_quotemeta)
PERL_PPDEF(Perl_pp_rv2av)
PERL_PPDEF(Perl_pp_aelemfast)
PERL_PPDEF(Perl_pp_aelem)
PERL_PPDEF(Perl_pp_aslice)
PERL_PPDEF(Perl_pp_each)
PERL_PPDEF(Perl_pp_values)
PERL_PPDEF(Perl_pp_keys)
PERL_PPDEF(Perl_pp_delete)
PERL_PPDEF(Perl_pp_exists)
PERL_PPDEF(Perl_pp_rv2hv)
PERL_PPDEF(Perl_pp_helem)
PERL_PPDEF(Perl_pp_hslice)
PERL_PPDEF(Perl_pp_unpack)
PERL_PPDEF(Perl_pp_pack)
PERL_PPDEF(Perl_pp_split)
PERL_PPDEF(Perl_pp_join)
PERL_PPDEF(Perl_pp_list)
PERL_PPDEF(Perl_pp_lslice)
PERL_PPDEF(Perl_pp_anonlist)
PERL_PPDEF(Perl_pp_anonhash)
PERL_PPDEF(Perl_pp_splice)
PERL_PPDEF(Perl_pp_push)
PERL_PPDEF(Perl_pp_pop)
PERL_PPDEF(Perl_pp_shift)
PERL_PPDEF(Perl_pp_unshift)
PERL_PPDEF(Perl_pp_sort)
PERL_PPDEF(Perl_pp_reverse)
PERL_PPDEF(Perl_pp_grepstart)
PERL_PPDEF(Perl_pp_grepwhile)
PERL_PPDEF(Perl_pp_mapstart)
PERL_PPDEF(Perl_pp_mapwhile)
PERL_PPDEF(Perl_pp_range)
PERL_PPDEF(Perl_pp_flip)
PERL_PPDEF(Perl_pp_flop)
PERL_PPDEF(Perl_pp_and)
PERL_PPDEF(Perl_pp_or)
PERL_PPDEF(Perl_pp_xor)
PERL_PPDEF(Perl_pp_cond_expr)
PERL_PPDEF(Perl_pp_andassign)
PERL_PPDEF(Perl_pp_orassign)
PERL_PPDEF(Perl_pp_method)
PERL_PPDEF(Perl_pp_entersub)
PERL_PPDEF(Perl_pp_leavesub)
PERL_PPDEF(Perl_pp_leavesublv)
PERL_PPDEF(Perl_pp_caller)
PERL_PPDEF(Perl_pp_warn)
PERL_PPDEF(Perl_pp_die)
PERL_PPDEF(Perl_pp_reset)
PERL_PPDEF(Perl_pp_lineseq)
PERL_PPDEF(Perl_pp_nextstate)
PERL_PPDEF(Perl_pp_dbstate)
PERL_PPDEF(Perl_pp_unstack)
PERL_PPDEF(Perl_pp_enter)
PERL_PPDEF(Perl_pp_leave)
PERL_PPDEF(Perl_pp_scope)
PERL_PPDEF(Perl_pp_enteriter)
PERL_PPDEF(Perl_pp_iter)
PERL_PPDEF(Perl_pp_enterloop)
PERL_PPDEF(Perl_pp_leaveloop)
PERL_PPDEF(Perl_pp_return)
PERL_PPDEF(Perl_pp_last)
PERL_PPDEF(Perl_pp_next)
PERL_PPDEF(Perl_pp_redo)
PERL_PPDEF(Perl_pp_dump)
PERL_PPDEF(Perl_pp_goto)
PERL_PPDEF(Perl_pp_exit)
PERL_PPDEF(Perl_pp_open)
PERL_PPDEF(Perl_pp_close)
PERL_PPDEF(Perl_pp_pipe_op)
PERL_PPDEF(Perl_pp_fileno)
PERL_PPDEF(Perl_pp_umask)
PERL_PPDEF(Perl_pp_binmode)
PERL_PPDEF(Perl_pp_tie)
PERL_PPDEF(Perl_pp_untie)
PERL_PPDEF(Perl_pp_tied)
PERL_PPDEF(Perl_pp_dbmopen)
PERL_PPDEF(Perl_pp_dbmclose)
PERL_PPDEF(Perl_pp_sselect)
PERL_PPDEF(Perl_pp_select)
PERL_PPDEF(Perl_pp_getc)
PERL_PPDEF(Perl_pp_read)
PERL_PPDEF(Perl_pp_enterwrite)
PERL_PPDEF(Perl_pp_leavewrite)
PERL_PPDEF(Perl_pp_prtf)
PERL_PPDEF(Perl_pp_print)
PERL_PPDEF(Perl_pp_sysopen)
PERL_PPDEF(Perl_pp_sysseek)
PERL_PPDEF(Perl_pp_sysread)
PERL_PPDEF(Perl_pp_syswrite)
PERL_PPDEF(Perl_pp_send)
PERL_PPDEF(Perl_pp_recv)
PERL_PPDEF(Perl_pp_eof)
PERL_PPDEF(Perl_pp_tell)
PERL_PPDEF(Perl_pp_seek)
PERL_PPDEF(Perl_pp_truncate)
PERL_PPDEF(Perl_pp_fcntl)
PERL_PPDEF(Perl_pp_ioctl)
PERL_PPDEF(Perl_pp_flock)
PERL_PPDEF(Perl_pp_socket)
PERL_PPDEF(Perl_pp_sockpair)
PERL_PPDEF(Perl_pp_bind)
PERL_PPDEF(Perl_pp_connect)
PERL_PPDEF(Perl_pp_listen)
PERL_PPDEF(Perl_pp_accept)
PERL_PPDEF(Perl_pp_shutdown)
PERL_PPDEF(Perl_pp_gsockopt)
PERL_PPDEF(Perl_pp_ssockopt)
PERL_PPDEF(Perl_pp_getsockname)
PERL_PPDEF(Perl_pp_getpeername)
PERL_PPDEF(Perl_pp_lstat)
PERL_PPDEF(Perl_pp_stat)
PERL_PPDEF(Perl_pp_ftrread)
PERL_PPDEF(Perl_pp_ftrwrite)
PERL_PPDEF(Perl_pp_ftrexec)
PERL_PPDEF(Perl_pp_fteread)
PERL_PPDEF(Perl_pp_ftewrite)
PERL_PPDEF(Perl_pp_fteexec)
PERL_PPDEF(Perl_pp_ftis)
PERL_PPDEF(Perl_pp_fteowned)
PERL_PPDEF(Perl_pp_ftrowned)
PERL_PPDEF(Perl_pp_ftzero)
PERL_PPDEF(Perl_pp_ftsize)
PERL_PPDEF(Perl_pp_ftmtime)
PERL_PPDEF(Perl_pp_ftatime)
PERL_PPDEF(Perl_pp_ftctime)
PERL_PPDEF(Perl_pp_ftsock)
PERL_PPDEF(Perl_pp_ftchr)
PERL_PPDEF(Perl_pp_ftblk)
PERL_PPDEF(Perl_pp_ftfile)
PERL_PPDEF(Perl_pp_ftdir)
PERL_PPDEF(Perl_pp_ftpipe)
PERL_PPDEF(Perl_pp_ftlink)
PERL_PPDEF(Perl_pp_ftsuid)
PERL_PPDEF(Perl_pp_ftsgid)
PERL_PPDEF(Perl_pp_ftsvtx)
PERL_PPDEF(Perl_pp_fttty)
PERL_PPDEF(Perl_pp_fttext)
PERL_PPDEF(Perl_pp_ftbinary)
PERL_PPDEF(Perl_pp_chdir)
PERL_PPDEF(Perl_pp_chown)
PERL_PPDEF(Perl_pp_chroot)
PERL_PPDEF(Perl_pp_unlink)
PERL_PPDEF(Perl_pp_chmod)
PERL_PPDEF(Perl_pp_utime)
PERL_PPDEF(Perl_pp_rename)
PERL_PPDEF(Perl_pp_link)
PERL_PPDEF(Perl_pp_symlink)
PERL_PPDEF(Perl_pp_readlink)
PERL_PPDEF(Perl_pp_mkdir)
PERL_PPDEF(Perl_pp_rmdir)
PERL_PPDEF(Perl_pp_open_dir)
PERL_PPDEF(Perl_pp_readdir)
PERL_PPDEF(Perl_pp_telldir)
PERL_PPDEF(Perl_pp_seekdir)
PERL_PPDEF(Perl_pp_rewinddir)
PERL_PPDEF(Perl_pp_closedir)
PERL_PPDEF(Perl_pp_fork)
PERL_PPDEF(Perl_pp_wait)
PERL_PPDEF(Perl_pp_waitpid)
PERL_PPDEF(Perl_pp_system)
PERL_PPDEF(Perl_pp_exec)
PERL_PPDEF(Perl_pp_kill)
PERL_PPDEF(Perl_pp_getppid)
PERL_PPDEF(Perl_pp_getpgrp)
PERL_PPDEF(Perl_pp_setpgrp)
PERL_PPDEF(Perl_pp_getpriority)
PERL_PPDEF(Perl_pp_setpriority)
PERL_PPDEF(Perl_pp_time)
PERL_PPDEF(Perl_pp_tms)
PERL_PPDEF(Perl_pp_localtime)
PERL_PPDEF(Perl_pp_gmtime)
PERL_PPDEF(Perl_pp_alarm)
PERL_PPDEF(Perl_pp_sleep)
PERL_PPDEF(Perl_pp_shmget)
PERL_PPDEF(Perl_pp_shmctl)
PERL_PPDEF(Perl_pp_shmread)
PERL_PPDEF(Perl_pp_shmwrite)
PERL_PPDEF(Perl_pp_msgget)
PERL_PPDEF(Perl_pp_msgctl)
PERL_PPDEF(Perl_pp_msgsnd)
PERL_PPDEF(Perl_pp_msgrcv)
PERL_PPDEF(Perl_pp_semget)
PERL_PPDEF(Perl_pp_semctl)
PERL_PPDEF(Perl_pp_semop)
PERL_PPDEF(Perl_pp_require)
PERL_PPDEF(Perl_pp_dofile)
PERL_PPDEF(Perl_pp_entereval)
PERL_PPDEF(Perl_pp_leaveeval)
PERL_PPDEF(Perl_pp_entertry)
PERL_PPDEF(Perl_pp_leavetry)
PERL_PPDEF(Perl_pp_ghbyname)
PERL_PPDEF(Perl_pp_ghbyaddr)
PERL_PPDEF(Perl_pp_ghostent)
PERL_PPDEF(Perl_pp_gnbyname)
PERL_PPDEF(Perl_pp_gnbyaddr)
PERL_PPDEF(Perl_pp_gnetent)
PERL_PPDEF(Perl_pp_gpbyname)
PERL_PPDEF(Perl_pp_gpbynumber)
PERL_PPDEF(Perl_pp_gprotoent)
PERL_PPDEF(Perl_pp_gsbyname)
PERL_PPDEF(Perl_pp_gsbyport)
PERL_PPDEF(Perl_pp_gservent)
PERL_PPDEF(Perl_pp_shostent)
PERL_PPDEF(Perl_pp_snetent)
PERL_PPDEF(Perl_pp_sprotoent)
PERL_PPDEF(Perl_pp_sservent)
PERL_PPDEF(Perl_pp_ehostent)
PERL_PPDEF(Perl_pp_enetent)
PERL_PPDEF(Perl_pp_eprotoent)
PERL_PPDEF(Perl_pp_eservent)
PERL_PPDEF(Perl_pp_gpwnam)
PERL_PPDEF(Perl_pp_gpwuid)
PERL_PPDEF(Perl_pp_gpwent)
PERL_PPDEF(Perl_pp_spwent)
PERL_PPDEF(Perl_pp_epwent)
PERL_PPDEF(Perl_pp_ggrnam)
PERL_PPDEF(Perl_pp_ggrgid)
PERL_PPDEF(Perl_pp_ggrent)
PERL_PPDEF(Perl_pp_sgrent)
PERL_PPDEF(Perl_pp_egrent)
PERL_PPDEF(Perl_pp_getlogin)
PERL_PPDEF(Perl_pp_syscall)
PERL_PPDEF(Perl_pp_lock)
PERL_PPDEF(Perl_pp_threadsv)
PERL_PPDEF(Perl_pp_setstate)
PERL_PPDEF(Perl_pp_method_named)
PERL_PPDEF(Perl_pp_dor)
PERL_PPDEF(Perl_pp_dorassign)
PERL_PPDEF(Perl_pp_entergiven)
PERL_PPDEF(Perl_pp_leavegiven)
PERL_PPDEF(Perl_pp_enterwhen)
PERL_PPDEF(Perl_pp_leavewhen)
PERL_PPDEF(Perl_pp_break)
PERL_PPDEF(Perl_pp_continue)
PERL_PPDEF(Perl_pp_smartmatch)
PERL_PPDEF(Perl_pp_say)

/* ex: set ro: */
